# Mattermost Integration module readme
The Mattermost Integration module has the ability to catch Outgoing Webhooks
from Mattermost and convert them into nodes, comments and files.

## TODO:
- Security:
  - Encrypt the Mattermost authentication token instead of storing it as a
    state.
- UX:
  - Automatically add "mattermost_username" field to users when installing
    module.
  - Automatically add "mattermost_post_id" field to content type when creating a
  webhook.
  - Automatically create "mattermost_file" fields and add to content and comment
  types when a webhook is created with these content/comment types.
  - Create a test endpoint where POST data is shown to user so user can check
    data.
  - Always respond *something* to the Mattermost channel and/or log a human
    readable error in the Watchdog.
- Possible bugs:
  - When a webhook is created with an existing Mattermost channel, a reply could
    be made on a post that hasn't gone through the webhook won't have a
    associated node, create the reply's parent node and create the comment on
    this newly created node.
  - Remove the Channel ID settings since the token is also unique per webhook
    and use the token to validate the type of content. Or replace Channel ID
    with the channel name so the site admin can easily fill this in (without
    having to request from the API).
  - Check for overlong titles and truncate them.
- Various:
  - Remove ```\Drupal``` calls.
- Features:
  - Support for editing posts.
  - Support for deleting posts.


## Configuration
On your Drupal site configure the "Mattermost authentication token" (see the
[Mattermost API](https://api.mattermost.com/#tag/authentication) for
information on how to obtain this token) and your "Mattermost server URL".

### Outgoing Webhooks
Point the Outgoing Webhook to your (publicly accessible) Drupal site at the
path: "/admin/config/services/mattermost-integration/endpoint?_format=json", 
make sure the content type on your Outgoing Webhook is "application/json". Next
create an Outgoing Webhook on your Drupal site. See the [Mattermost API](https://api.mattermost.com/#tag/channels%2Fpaths%2F~1teams~1%7Bteam_id%7D~1channels~1name~1%7Bchannel_name%7D%2Fget)
for information about obtaining the Channel ID. The webhook token is generated
by Mattermost upon creation of an Outgoing Webhook (in Mattermost) and is used
to verify the origin of a incoming request. If the token does not match the
request content will not be created.

### Users
For the Mattermost Integration module to post content under the correct user, a
"mattermost_username" field must be created on the users. This field must be
filled in manually for each user.

The "mattermost_username" field must be a "Text (plain)" field with the
machine-readable name being "field_mattermost_username". The fields' maximum
length must be at least **64** characters. More is allowed but the maximum
allowed by Mattermost is 64 so more characters aren't needed. The field must
have a limit of **1** on the allowed number of values.

When a request is received and no matching user is found the node/comment will
be published under the "Anonymous" user account (UID: 0).

### Content
A "mattermost_post_id" field must be available on the content type(s) webhooks
can create.

The "mattermost_post_id" field must be a "Text (plain)" field with its'
machine-readable name being "field_mattermost_post_id". The fields' maximum
length must be at least **26** characters with a limit of **1** on the allowed
number of values.

### Comments
Comments themselves don't require any custom fields, but what they do require is
a comment field on the target entity. For example: If you have an outgoing
webhook that creates content of type "Article", then the "Article" content type
must have a field of the type "Comments".

### Files
If you want to be able to attach files to a node make sure there is a 
"mattermost_file" field available in your content type and comment type.

The "mattermost_file" field must be a "File field" with the machine-readable 
name being "field_mattermost_file". Its upload destination must be public and 
have an unlimited allowed amount of values.

The allowed file extensions are ignored by the module, but if you wish to edit 
the content manually later you will have to fill this in or the form will 
prevent you from saving.
