<?php

namespace Drupal\mattermost_integration\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class MattermostIntegrationConfigurationForm Global configuration form.
 *
 * @package Drupal\mattermost_integration\Form
 */
class MattermostIntegrationConfigurationForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mattermost_integration_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'mattermost_integration.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('mattermost_integration.settings');

    $form['token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mattermost authentication token'),
      '#description' => $this->t('This is the token used to validate against the Mattermost API using the Bearer method.'),
      // @codingStandardsIgnoreStart
      '#default_value' => \Drupal::state()->get('mattermost_authentication_token'),
      // @codingStandardsIgnoreEnd
      '#maxlength' => 26,
      '#size' => 26,
      '#required' => TRUE,
    ];
    $form['api_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Mattermost server URL'),
      '#description' => $this->t('Enter your Mattermost server URL where the API can be found.'),
      '#default_value' => $config->get('api_url'),
      '#required' => TRUE,
      '#field_suffix' => '/api/v3/',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    if (preg_match('/^http?:\/\//', $values['api_url'])) {
      drupal_set_message($this->t('Warning: you are using an insecure connection for your Mattermost server (HTTP). It is strongly recommended to use a secured connection (HTTPS)!'), 'warning');
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * @TODO: Encrypt the Mattermost authentication token instead of storing it as
   *        a state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $this->config('mattermost_integration.settings')
      ->set('api_url', $values['api_url'])
      ->save();

    // Save the token in a state so it won't be exported in plain text YML.
    // @codingStandardsIgnoreStart
    \Drupal::state()->set('mattermost_authentication_token', $values['token']);
    // @codingStandardsIgnoreEnd

    parent::submitForm($form, $form_state);
  }

}
